/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * SCIDYNE Corporation, 649 School Street, Pembroke MA 02359 USA
 * File: dio64-ard_Adafruit_demo.ino
 * Compiler: Arduino IDE Ver 1.8.10
 * Date: December 6, 2022
 * Author: Mark Durgin 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * 
 * This sketch demonstrates how to use the SCIDYNE DIO64-ARD (PN #100-7728) with the Adafruit 
 * open-source MCP23017 library (Version 2.02). SCIDYNE is not affiliated with Adafruit but their
 * contributions and products are appreciated. Please support Adafruit by visiting www.adafruit.com
 * 
 * The Adafruit MCP23017 library must be available within the Arduino IDE. If not installed already, 
 * go to Sketch > Include Library > Manage Libraries. Search for MCP23017 when prompted and follow 
 * the installation instructions.
 *  
 * Summary of the library routines:
 * // Initialization
 * bool begin_I2C( uint8_t i2c_addr );
 *
 * // Main Arduino API methods
 * void pinMode( uint8_t pin, uint8_t mode );
 * uint8_t digitalRead( uint8_t pin );
 * void digitalWrite( uint8_t pin, uint8_t value );
 *
 * // Bulk access
 * uint8_t readGPIO( uint8_t port );
 * void writeGPIO( uint8_t value, uint8_t port );
 *
 * // Interrupts
 * void setupInterrupts( bool mirroring, bool openDrain, uint8_t polarity );
 * void setupInterruptPin( uint8_t pin, uint8_t mode );
 * void disableInterruptPin( uint8_t pin );
 * uint8_t getLastInterruptPin();
 * 
 * Library Usage Hints:
 *  - Values for Pin parameters are 0 - 15 corresponding to the 8-bits of a PortA (Pins 0 - 7) and 
 *    the 8-Bits of a PortB (Pins 8 - 15).
 *  - Values for Port parameters are 0 for portA and 1 for PortB
 * 
 * For clarity, only the basic steps necessary to include, setup and start using the library with 
 * the DIO64-ARD are shown. Please refer to the actual library files and other documentation for
 * more thorough library information.
 * 
 * The program continuously generates pulses on certain Pins as defined within the software. It also 
 * reads the 8-bit PortH, increments the value, and then writes the new value back to PortH.
 * 
 * Program operation can be observed using the Arduino IDE Serial Monitor and by oscilloscope.
 * 
 */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Include necessary files
 */
#include <Adafruit_MCP23X17.h>    // Include the Adafruit MCP23017 library


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Define the I2C addresses of the four MCP23017 chips used on the DIO64-ARD.
 * Make sure no other I2C devices use these addresses that would cause a conflict.
 * If the address offset jumper (SJ8) on the DIO64-ARD is enabled be sure to
 * add 0x04 to each of the addresses, resulting in addresses 0x24 - 0x27. 
 */
#define DIO64ARD_U1PAPB_ADDR  0x20    // I2C Address of U1, associated with DIO64-ARD PortA and PortB
#define DIO64ARD_U2PCPD_ADDR  0x21    // I2C Address of U2, associated with DIO64-ARD PortC and PortD (i.e; U2 PortA and PortB respectfully)
#define DIO64ARD_U3PEPF_ADDR  0x22    // I2C Address of U3, associated with DIO64-ARD PortE and PortF (i.e; U3 PortA and PortB respectfully)
#define DIO64ARD_U4PGPH_ADDR  0x23    // I2C Address of U4, associated with DIO64-ARD PortG and PortH (i.e; U4 PortA and PortB respectfully)

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Create four instances of Adafruit library, one for each MCP23017 chip on the DIO64-ARD board
 * As an alternative, the software could be structured to use an array of the library
 * such as    Adafruit_MCP23X17 DIO64ARD[4];
 */
Adafruit_MCP23X17 DIO64ARD_U1PAPB;    // Create instance of library to control U1
Adafruit_MCP23X17 DIO64ARD_U2PCPD;    // Create instance of library to control U2
Adafruit_MCP23X17 DIO64ARD_U3PEPF;    // Create instance of library to control U3
Adafruit_MCP23X17 DIO64ARD_U4PGPH;    // Create instance of library to control U4


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Define program constants
 */
#define PORTA_PIN     0   // The PortA Pin to toggle
#define PORTA_PULSCNT 1   // Number of pulses to generate on the designated PortA Pin

#define PORTC_PIN     0   // The PortC Pin to toggle
#define PORTC_PULSCNT 2   // Number of pulses to generate on the designated PortC Pin

#define PORTE_PIN     0   // The PortE Pin to toggle
#define PORTE_PULSCNT 3   // Number of pulses to generate on the designated PortE Pin

#define PORTG_PIN     0   // The PortG Pin to toggle
#define PORTG_PULSCNT 4   // Number of pulses to generate on the designated PortG Pin


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Setup runs once then execution goes to loop
 */
void setup()
{
  uint8_t i;  // A general purpose temporary variable
  
  Serial.begin(115200); // Setup communications to IDE Monitor
    while (!Serial);

  Serial.println(F("SCIDYNE DIO64-ARD using Adafruit MCP23017 Library Demo"));

  /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   * Initialize the library associated with U1, DIO64-ARD PortA and PortB
   */ 
  if (!DIO64ARD_U1PAPB.begin_I2C(DIO64ARD_U1PAPB_ADDR)) {
    Serial.println(F("Error initializing U1"));
    while (1);
  }


  /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   * Initialize the library associated with U2, DIO64-ARD PortC and PortD
   * PortC is physically PortA of U2,  PortD is physically PortB of U2
   */
  if (!DIO64ARD_U2PCPD.begin_I2C(DIO64ARD_U2PCPD_ADDR)) {
    Serial.println(F("Error initializing U2"));
    while (1);
  }

  
  /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   * Initialize the library associated with U3, DIO64-ARD PortE and PortF
   * PortE is physically PortA of U3,  PortF is physically PortB of U3
   */
  if (!DIO64ARD_U3PEPF.begin_I2C(DIO64ARD_U3PEPF_ADDR)) {
    Serial.println(F("Error initializing U3"));
    while (1);
  }


  /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   * Initialize the library associated with U4, DIO64-ARD PortG and PortH
   * PortG is physically PortA of U4,  PortH is physically PortB of U4
   */
  if (!DIO64ARD_U4PGPH.begin_I2C(DIO64ARD_U4PGPH_ADDR)) {
    Serial.println(F("Error initializing U4"));
    while (1);
  }

  /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   * Configure the designated pins on PortA, PortC, PortE, and PortG
   * Where mode can be:
   *  OUTPUT            Configures Pin as an output 
   *  INPUT             Configures Pin as Input
   *  INPUT_PULLUP      Configures Pin as input with Pull-Up enabled
   */
  DIO64ARD_U1PAPB.pinMode(PORTA_PIN, OUTPUT);
  DIO64ARD_U2PCPD.pinMode(PORTC_PIN, OUTPUT);
  DIO64ARD_U3PEPF.pinMode(PORTE_PIN, OUTPUT);
  DIO64ARD_U4PGPH.pinMode(PORTG_PIN, OUTPUT);

  /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   * Setup PortH as all outputs, done by making each bit an output.
   * PortH is actually PortB of U3 and is treated as Pins 8 - 15 by the library
   */
  for (i=8; i < 16; i++)
  {
    DIO64ARD_U4PGPH.pinMode(i, OUTPUT);   // Set bit direction to output 
  }

  Serial.println(F("Looping..."));  // Tell user program is running

} /* End Setup */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * loop runs continuously
 */
void loop()
{
  uint8_t i;                    // General purpose variable
  static uint8_t loopcntr = 0;  // Loop counter
  
  // Generate pulses on designated portA pin
  for (i=0; i < PORTA_PULSCNT; i++)
  {
    DIO64ARD_U1PAPB.digitalWrite(PORTA_PIN, HIGH);
    //delay(1);
    DIO64ARD_U1PAPB.digitalWrite(PORTA_PIN, LOW);
    //delay(1);
  }

  
  // Generate pulses on designated portC pin
  for (i=0; i < PORTC_PULSCNT; i++)
  {
    DIO64ARD_U2PCPD.digitalWrite(PORTC_PIN, HIGH);
    //delay(1);
    DIO64ARD_U2PCPD.digitalWrite(PORTC_PIN, LOW);
    //delay(1);
  }

  // Generate pulses on designated portE pin
  for (i=0; i < PORTE_PULSCNT; i++)
  {
    DIO64ARD_U3PEPF.digitalWrite(PORTE_PIN, HIGH);
    //delay(1);
    DIO64ARD_U3PEPF.digitalWrite(PORTE_PIN, LOW);
    //delay(1);
  }

  // Generate pulses on designated portG pin
  for (i=0; i < PORTG_PULSCNT; i++)
  {
    DIO64ARD_U4PGPH.digitalWrite(PORTG_PIN, HIGH);
    //delay(1);
    DIO64ARD_U4PGPH.digitalWrite(PORTG_PIN, LOW);
    //delay(1);
  }



  // Write the 8-bit loop counter value to PortH (which is actually PortB of U4)
  DIO64ARD_U4PGPH.writeGPIO( loopcntr, 1 );

  // Read 8-bit PortH value and display it
  i = DIO64ARD_U4PGPH.readGPIO( 1 ); 
  Serial.print(F("PortH: "));
  Serial.println(i,HEX);

  loopcntr++;   // Increment the loop counter


}  /* End Loop */
